import '@testing-library/jest-dom/vitest';
import { describe, expect, test } from 'vitest';
import { Board, Difficulty } from './board';

describe('Board', () => {
	test('shape', () => {
		const board = new Board();

		expect(board.grid.length).toBe(3);
		expect(board.grid[0].length).toBe(3);
		expect(board.horizontalSum.length).toBe(3);
		expect(board.verticalSum.length).toBe(3);
	});

	test('isComplete', () => {
		const board = new Board();
		expect(board.isComplete()).toBe(true);

		// remove one number
		board.setByPos(3);
		expect(board.isComplete()).toBe(false);
	});

	test('isValid', () => {
		const board = new Board();
		expect(board.isValid()).toBe(true);
	});

	test('isValid ko', () => {
		const board = new Board();
		board.horizontalSum[1] = -1;
		expect(board.isValid()).toBe(false);
	});

	test('setByPos', () => {
		const board = new Board();
		// set board from 9 to 1
		for (let index = 0; index < 9; index++) {
			board.setByPos(index, 9 - index);
		}
		expect(board.grid[0][0]).toBe(9);
		expect(board.grid[0][2]).toBe(7);
		expect(board.grid[1][0]).toBe(6);
		expect(board.grid[1][1]).toBe(5);
		expect(board.grid[2][2]).toBe(1);
	});

	test('prepare', () => {
		const board = new Board();
		board.prepare(Difficulty.Easy);

		let emptyCellNb = 0;
		for (const cell of board) {
			if (cell === undefined) emptyCellNb++;
		}
		expect(emptyCellNb).toBe(9 - 3);
	});
});
